![school](asset/hitsz-logo.jpg)

# MankorOS

## 项目简述

MankorOS 是 [Rust](https://www.rust-lang.org) 编写的基于 [RISC-V](https://riscv.org) 的操作系统。

## 完成情况

初赛满分 (2023-05-23)
![leaderboard-pre](asset/leaderboard-pre.png)

## MankorOS 内核介绍

- 无栈协程基建: 基于全局队列实现的调度器, 可供异步程序执行
- 内存管理: 实现 mmap/munmap 系统调用, 可对所有内存段进行懒分配或懒加载, 具备写时复制 (CoW) 功能
- 文件系统: 完成虚拟文件系统, 支持 devfs 和管道
- 进程管理: 支持 clone 系统调用, 可以细粒度划分进程共享的资源
- 信号机制: 完成基础的信号机制
- 用户程序: 能通过所有初赛测试样例

## 开发文档

文档的二进制存放于 [doc](doc) 目录，开发文档的源码与模版位于 [github:mankoros/mankoros-doc](https://github.com/mankoros/mankoros-doc)。

[初赛开发文档](doc/初赛开发文档.pdf)

## 项目结构

```shell
.
├── src
│  ├── arch         # 汇编与平台相关的包装函数
│  ├── axerrno      # 错误处理
│  ├── consts       # 常量
│  │  └── platform
│  ├── driver       # 驱动，包括块设备和 uart 驱动
│  ├── executor     # 管理 future 执行
│  ├── fs           # 文件系统
│  │  ├── devfs     # 设备文件系统
│  │  └── vfs       # 虚拟文件系统
│  ├── memory       # 内存
│  │  ├── address   # 地址类型
│  │  └── pagetable # 页表
│  ├── process      # 进程
│  │  └── user_space
│  ├── signal       # 信号
│  ├── sync         # 同步，一致性、锁和进程通讯
│  ├── syscall      # POSIX 系统调用
│  ├── timer        # 时钟，时钟中断管理和计时器
│  ├── tools        # 工具类型和函数
│  ├── trap         # 中断
│  ├── xdebug       # debug 工具
│  ├── boot.rs      # 启动
│  ├── lazy_init.rs # 懒加载
│  ├── logging.rs   # 日志
│  ├── main.rs      
│  └── utils.rs
├── vendor
├── Cargo.toml
├── LICENSE
├── linker.ld
├── Makefile
├── README.md
└── rust-toolchain.toml
```

## 如何运行

编译内核：

```shell
make all
```

使用 `qemu-system-riscv64` 测试：

```shell
make release-qemu
```

## 项目人员

哈尔滨工业大学（深圳）:
- 满洋 (manyang.me@outlook.com)
- 梁韬 (Origami404@foxmail.com)
- 苏亦凡 (sorashu@outlook.com)
- 指导老师：夏文，仇洁婷

## 参考

- [rcore-os/rCore](https://github.com/rcore-os/rCore)
- [rcore-os/arceos](https://github.com/rcore-os/arceos)
- [FTL OS](https://gitlab.eduxiji.net/DarkAngelEX/oskernel2022-ftlos)
- [Maturin](https://github.com/scPointer/maturin)


## 鸣谢

本项目使用和参考了许多开源项目，其中包括 [RustSBI](https://github.com/rustsbi/rustsbi)，[bitmap-allocator](https://github.com/rcore-os/bitmap-allocator)，[virtio-drivers](https://github.com/rcore-os/virtio-drivers) 等。
感谢这些项目的开发者为开源社区做出的贡献以及对本项目开发的启发和帮助。

感谢本校学长叶自立在设计上给出的指导。感谢两位指导老师和同校其它参赛队伍在项目开发过程中的帮助。

## 开源协议

本项目采用 GPLv2 或更高版本的 GPL 协议。